<?php

/**
 * Author：晓风
 * Creation：2024/08/31 17:36
 * Filename：hwd.php
 * 恨晚岛
 * QQ交流群960640976
 */

namespace lib\supply;

use DateTime;
use Medoo\DB\SQL;

class hwd
{
    /**
     * @var array
     * 对接配置信息
     */
    public static $Config = [
        'name' => '恨晚岛',
        'image' => '../assets/img/hwd.ico',
        'PriceMonitoring' => 1, //是否支持价格监控
        'BatchShelves' => 1, //批量上架
        'help' => '恨晚岛对接插件,本插件开源免费使用,作者晓风,QQ3295564195，QQ交流群960640976！',
        'ip' => 1,
        'field' => [ //添加货源的字段名称
            'url' => [
                'name' => '接口地址',
                'tips' => '请填写包含http(s):// 和 / 的接口地址!',
                'type' => 1,
            ],
            'username' => [
                'name' => '用户名',
                'tips' => '请填写用户名!',
                'type' => 1,
            ],
            'password' => [
                'name' => 'token',
                'tips' => '请填写秘钥token!',
                'type' => 1,
            ],
        ],
        'InputField' => [ //选择对接商品时的输入框字段
            'ProductList' => [
                'type' => 2,
                'name' => '可对接商品', //下拉列表名称
                'request' => [ //提交的是当前选择参数数据类的值 + 当前的货源社区ID加类型键值！
                    'gid' => false,
                    'type' => 1, //固定提交的参数
                    'controller' => 'ProductDetails', //请求的控制器名称
                ],
                'GetListData' => [ //获取列表时的请求数据，提交的是当前所选对接社区内的值+当前已选参数的值
                    'id' => false,
                    'type' => 1,
                    'controller' => 'GoodsList', //请求的控制器名称
                ],
                'RequestDataSources' => '-1', //点击列表参数请求替换源，data() { return (全部可用) }，-1为内置源GoodsData，其他为App[xxx]
                'GetListDataSources' => 'DockingData', //列表请求替换源，data() { return (全部可用) }
                'BtnName' => '获取数据', //获取列表时的按钮名称
                'button' => [ //辅助按钮
                    [
                        'name' => '获取最新数据',
                        'GetListDataSources' => 'DockingData', //请求替换源，data() { return (全部可用) }
                        'class' => 'badge badge-primary-lighten',
                        'request' => [
                            'id' => false,
                            'type' => 2,
                            'controller' => 'GoodsList', //请求的控制器名称
                        ],
                    ]
                ]
            ],
            'gid' => [
                'type' => 1,
                'name' => '商品ID',
                'reminder' => '请点击获取数据后，选择对应商品获取'
            ],
            'parameter' => [
                'type' => 1,
                'name' => '对接参数',
                'reminder' => '请点击获取数据后，选择对应商品获取'
            ]
        ]
    ];

    /**
     * @param $Data //用户提交的数据
     * @param $Source //货源详情
     * @param $DataSo //提交按钮详情
     */
    public static function AdminOrigin($Data = [], $Source = [], $DataSo = [])
    {
        $plugin_id = 11;
        $plugin_name = "恨晚岛对接插件";
        switch ($Data['controller']) {
            case 'GoodsList': //获取可对接的商品列表
                return self::GoodsList($Data);
            case 'ProductDetails': //获取商品详情
                return self::ProductDetails($Data);
            case 'Submit': //对接下单
                return self::Submit($Data['Order'], $Data['Goods'], $Data['Supply']);
            case 'Query': //查询订单
                return self::Query($Data['Order'], $Data['Supply']);
            case 'CommodityStatus': //商品状态详情
                return self::CommodityStatus($Data['Goods'], $Data['Supply']);
            case 'CommodityAnalysis': //解析商品配置信息
                return self::CommodityAnalysis($Data);
        }
        return [
            'code' => -1,
            'msg' => '请求错误',
        ];
    }

    /**
     * @param $Data
     * @return array
     * 获取可对接商品列表
     */
    public static function GoodsList($Data)
    {
        if ($Data['type'] != 2 && !empty($_SESSION['hwdGoodsList_' . $Data['id']])) {
            return [
                'code' => 1,
                'msg' => '可对接商品列表获取成功！',
                'data' => $_SESSION['hwdGoodsList_' . $Data['id']]
            ];
        }

        $DB = SQL::DB();
        $SourceData = $DB->get('shequ', ['url', 'username', 'password'], [
            'id' => $Data['id'],
        ]);
        if (!$SourceData) {
            return [
                'code' => -1,
                'msg' => '对接社区数据获取失败，请检查此对接社区是否存在！',
            ];
        }
        $requestURI = '/api/hw/product.php';
        $postData = [
            'token' => $SourceData['password'],
        ];
        $SourceData['url'] = StringCargo::UrlVerify($SourceData['url'], 3);
        $CurlData = Api::Curl($SourceData['url'] . $requestURI, $postData);
        $CurlDataJson = json_decode($CurlData, true);
        if (empty($CurlDataJson)) {
            return [
                'code' => -1,
                'msg' => '对接网站：' . $SourceData['url'] . ' 打开失败，请检查域名是否有误，或手动Ping测试连通率',
            ];
        }
        // die(json_encode($CurlDataJson, JSON_UNESCAPED_UNICODE));
        if ($CurlDataJson['code'] === 1) {
            $GoodsList = [];
            foreach ($CurlDataJson['data'] as $v) {
                $GoodsList[] = [
                    'gid' => $v['id'],
                    'name' => $v['name'] . ' - ' . ($v['active'] === 1 ? '正常对接' : '关闭下单'),
                    'price' => $v['price'],
                    'type' => $v['type'],
                    'active' => $v['active']
                ];
            }
            $_SESSION['hwdGoodsList_' . $Data['id']] = $GoodsList;
            return [
                'code' => 1,
                'msg' => '可对接商品列表获取成功！',
                'data' => $GoodsList
            ];
        }

        return [
            'code' => -1,
            'msg' => $CurlDataJson['msg'],
        ];
    }

    /**
     * @param $array
     * @return string
     * 直客签名
     */
    public static function GetSign($param, $key)
    {
        $signPars = '';
        ksort($param);
        foreach ($param as $k => $v) {
            $k = trim($k);
            $v = trim($v);
            if ($k !== 'sign' && $v !== '') {
                $signPars .= $k . '=' . $v . '&';
            }
        }
        $signPars = trim($signPars, '&');
        $signPars .= $key;
        return md5($signPars);
    }

    /**
     * @param $Data
     * @return array
     * 获取新增商品SQL数据
     */
    public static function CommodityAnalysis($Data)
    {
        global $date;
        $Goods = json_decode($Data['data'], true);
        $SQL = [
            'cid' => $Data['cid'],
            'sort' => $Data['sort'],
            'name' => $Data['name'],
            'image[JSON]' => [ImageUrl($Goods['image'])],
            'min' => $Goods['min'],
            'max' => $Goods['max'],
            'quota' => $Goods['quota'],
            'input' => $Goods['input'],
            'quantity' => $Goods['quantity'],
            'alert' => $Goods['alert'],
            'docs' => $Goods['docs'],
            'money' => $Goods['money'],
            'units' => (empty($Goods['units']) ? '个' : $Goods['units']),
            'deliver' => -1,
            'sqid' => $Data['sqid'],
            'specification' => 1,
            'specification_type' => 2,
            'specification_spu[JSON]' => [],
            'specification_sku[JSON]' => [],
            'extend[JSON]' => $Goods['extend'],
            'date' => $date,
        ];
        return $SQL;
    }


    /**
     * @param $Goods
     * @param $Supply
     * 商品详细状态
     * @return array
     */
    public static function CommodityStatus($Goods, $Supply)
    {
        $GoodsData = json_decode($Goods['extend'], true);
        if (empty($GoodsData['gid'])) {
            return [
                'code' => -1,
                'msg' => '此商品对接参数缺失，无法完成商品状态监控！',
            ];
        }

        $Supply['url'] = StringCargo::UrlVerify($Supply['url'], 3);
        $requestURI = '/api/hw/product.php?';
        $postData = [
            'id' => $GoodsData['gid'],
            'token' => $Supply['password'],
        ];
        $CurlData = Api::Curl($Supply['url'] . $requestURI, $postData);
        $CurlDataJson = json_decode($CurlData, true);
        if (empty($CurlDataJson)) {
            return [
                'code' => -1,
                'msg' => '对接网站：' . $Supply['url'] . ' 打开失败，请检查域名是否有误，或手动Ping测试连通率',
            ];
        }
        if ((int)$CurlDataJson['code'] != 1 || (int)$CurlDataJson['data'][0]['active'] != 1) {
            return [
                'code' => 1,
                'msg' => '数据获取成功',
                'data' => [
                    'state' => 2,
                    'money' => 0,
                    'inventory' => 0,
                ]
            ];
        }
        $GoodsDataApi = $CurlDataJson['data'][0];
        $goodsPrice = 0;
        if (intval($GoodsDataApi['type']) == 1) {
            $goodsPrice = $GoodsDataApi['price'];
        } else {
            $goodsPrice = $GoodsDataApi['price'] / 1000;
        }
        /**
         * 解析数据
         */
        return [
            'code' => 1,
            'msg' => '数据获取成功',
            'data' => [
                'inventory' => is_numeric($GoodsDataApi['count']) ? $GoodsDataApi['count'] : 99999,
                'state' => 1,
                'money' => $goodsPrice,
                'specification' => false,
            ]
        ];
    }

    /**
     * @param $Data
     * @return array
     * 获取商品详情
     */
    public static function ProductDetails($Data)
    {
        $DB = SQL::DB();
        $SourceData = $DB->get('shequ', ['url', 'username', 'password'], [
            'id' => $Data['sqid'],
        ]);
        if (!$SourceData) {
            return [
                'code' => -1,
                'msg' => '对接社区数据获取失败，请检查此对接社区是否存在！',
            ];
        }
        $SourceData['url'] = StringCargo::UrlVerify($SourceData['url'], 3);
        $requestURI = '/api/hw/product.php?';
        $postData = [
            'id' => $Data['gid'],
            'token' => $SourceData['password'],
        ];
        $CurlData = Api::Curl($SourceData['url'] . $requestURI, $postData);

        $CurlDataJson = json_decode($CurlData, true);
        if (empty($CurlDataJson)) {
            return [
                'code' => -1,
                'msg' => '对接网站：' . $SourceData['url'] . ' 打开失败，请检查域名是否有误，或手动Ping测试连通率',
            ];
        }
        if ($CurlDataJson['code'] !== 1) {
            return [
                'code' => -1,
                'msg' => $CurlDataJson['msg']
            ];
        }
        // die(json_encode($CurlDataJson,JSON_UNESCAPED_UNICODE));
        $Goods = $CurlDataJson['data'][0];
        if ($Goods['active'] !== 1) {
            //            return [
            //                'code' => -1,
            //                'msg' => '此商品已关闭下单！',
            //            ];
        }

        $Input = [];
        $Input[0][] = "下单邮箱";
        $Input[1][] = "email";

        $Image = $Goods['img'];
        if (!strstr($Image, 'http')) {
            $Image = $SourceData['url'] . $Image;
        }
        $goodsPrice = 0;
        if (intval($Goods['type']) == 1) {
            $goodsPrice = $Goods['price'];
        } else {
            $goodsPrice = $Goods['price'] / 1000;
        }
        return [
            'code' => 1,
            'msg' => '对接参数自动填写成功！',
            'data' => [
                'name' => $Goods['name'],
                'image' => $Image,
                'alert' => $Goods['description'],
                'docs' => $Goods['description'],
                'money' => $goodsPrice,
                'min' => 1,
                'max' => 999,
                'quota' =>  intval($Goods['count']),
                'quantity' => 1,
                'extend' => [
                    'gid' => $Goods['id'],
                    'parameter' => implode(',', $Input[1]),
                ],
                'units' => '个',
                'input' => implode('|', $Input[0]),
            ]
        ];
    }

    /**
     * @param $OrderData 订单信息
     * @param $Goods 商品信息
     * @param $TypeSupply 对接货源信息
     * 提交下单信息！
     */
    public static function Submit($OrderData, $Goods, $TypeSupply)
    {
        global $conf;
        $InputArray = json_decode($OrderData['input'], TRUE);
        if ($Goods['specification'] == 2 && $Goods['specification_type'] == 2) {
            $InputArray = RuleSubmitParameters(json_decode($Goods['specification_spu'], TRUE), $InputArray);
        }

        $GoodsData = json_decode($Goods['extend'], true); //对接数据
        $Field = explode(',', $GoodsData['parameter']);
        $requestURI = '/api/hw/dj.php?token=' . $TypeSupply['password'] . "&id=" . $GoodsData['gid'] . "&count=" . $OrderData['num']."&type=".$_SERVER['HTTP_HOST'];
        $TypeSupply['url'] = StringCargo::UrlVerify($TypeSupply['url'], 3);
        $DataCurl = Api::SuppluCurl($TypeSupply['url'] . $requestURI, []);
        if (empty($DataCurl)) {
            return [
                'code' => $conf['SubmitState'],
                'docking' => 2,
                'msg' => '对接返回信息有误：' . curl_error($ch),
                'money' => 0,
                'order' => 0,
            ];
        }
        // die($DataCurl);
        $DataCurl = json_decode($DataCurl, TRUE);
        if (!isset($DataCurl['code'])) {
            $Msg = '下单信息获取失败!,请检查对接配置！';
            $code = $conf['SubmitState'];
            $money = 0;
            $order = -1;
            $docking = 2;
        } else {
            $Msg = $DataCurl['msg'];
            $code = ($DataCurl['code'] === 0 ? $conf['SubmitStateSuccess'] : $conf['SubmitState']);
            $money = self::MoneyGet($TypeSupply) ?? -1;
            $order = -1;
            $docking = ($DataCurl['code'] === 0 ? 1 : 2);
        }
        if ($DataCurl['code'] === 0) {
            //同步卡密库存
            $OrderData['kami'] = $DataCurl['kami'];
            self::Query($OrderData, $TypeSupply);
        }

        return [
            'code' => $code,
            'docking' => $docking,
            'msg' => $Msg,
            'money' => $money,
            'order' => $order,
        ];
    }
    /**
     * 生成指定长度的随机字符串
     */
    private static function randomStr($num)
    {
        $str = '';
        for ($i = 0; $i < $num; $i++) {
            $str .= chr(mt_rand(97, 122));
        }
        return $str;
    }

    /**
     * 查询余额
     */
    public static function MoneyGet($TypeSupply)
    {
        $requestURI = '/api/cx/info.php';
        $posdData = [
            'keyword' => $TypeSupply['username'],
        ];
        $posdData = http_build_query($posdData);
        $TypeSupply['url'] = StringCargo::UrlVerify($TypeSupply['url'], 3);
        $DataCurl = Api::Curl($TypeSupply['url'] . $requestURI, $posdData,);
        $DataCurl = json_decode($DataCurl, true);
        if (empty($DataCurl) || $DataCurl['success'] != 'success') {
            return false;
        }
        $data = $DataCurl['data'][0];
        $balance = ($data['credit'] / 1000) + ($data['balance']);
        return $balance;
    }

    /**
     * @param $Order 订单信息
     * @param $TypeSupply 对接货源信息
     */
    public static function Query($Order, $TypeSupply)
    {
        $order_state = '未知状态';
        $StateNum = 3;
        $TokenList = $Order['kami'];
        if (isset($Order['kami']) && count($Order['kami']) >= 1) {
            $TokenList = $Order['kami'];
            //有卡
            $DB = SQL::DB();
            $TokenGet = $DB->count('token', [
                'order' => $Order['order'],
            ]) - 0;
            if ($TokenGet < count($TokenList)) {
                global $date;
                //补卡
                $SQL = [];
                foreach ($TokenList as $v) {
                    if (empty($v)) {
                        continue;
                    }
                    $SQL[] = [
                        'uid' => $Order['uid'],
                        'gid' => $Order['gid'],
                        'code' => json_decode($Order['input'], true)[0],
                        'token' => $v,
                        'ip' => $Order['ip'],
                        'order' => $Order['order'],
                        'endtime' => $date,
                        'addtime' => $date,
                    ];
                }
                if (count($SQL) >= 1) {
                    $DB->insert('token', $SQL);
                }
            }
        }
        $res = [
            'ApiType' => 1,
            'ApiNum' => $Order['num'],
            'ApiTime' => $Order['addtitm'],
            'ApiInitial' => 0,
            'ApiPresent' =>  $Order['num'],
            'ApiState' => $order_state,
            'ApiError' => "不支持订单状态查询",
            'ApiStateNum' => 1,
        ];
        return $res;
    }
}
